'''
Created on Oct 1, 2013

@author: afried

How to run this script:
* See README.txt!
'''
import os
import sys
import re
import codecs
from shutil import rmtree
from xml.dom.minidom import parse

def removeDir(path):
    if os.path.isdir(path):
        rmtree(path)
        
def getFilenames(path):
    files = []
    for f in os.listdir(path):
        if os.path.isdir(f):
            for g in getFilenames(f):
                files.append(f + "/" + g)
        else:
            files.append(f)
    return files
        

def mergeDataAndAnnotations(dataSourceDir, annotSourceDir, targetDir, subset):
    
    print("\nMerging data and annotations:")
    print("data:\t\t", dataSourceDir)
    print("annotations:\t", annotSourceDir)
    print("to:\t\t", targetDir)
    
    if os.path.exists(targetDir):
        removeDir(targetDir)
    os.makedirs(targetDir)
    
    for filename in getFilenames(annotSourceDir):
        # read file from data source directory
        if subset == "gflow":
            filename2 = re.sub("xml", "1", filename)
        else:
            filename2 = re.sub(".xml", "", filename)
        
        with codecs.open(dataSourceDir + "/" + filename2, "r", encoding="ISO-8859-1") as f:
            # read the summary's text
            text = ""
            for line in f:
                text += line.strip() + " "
            # read the annotations + add text
            domObj = parse(annotSourceDir + "/" + filename)
            # create text child node
            textChild = domObj.createElement("TEXT")
            textNode = domObj.createCDATASection("\n" + text + "\n")
            textChild.appendChild(textNode)
            tagChild = domObj.getElementsByTagName("TAGS")[0]
            schemeChild = domObj.getElementsByTagName("annotationScheme")
            schemeChild[0].insertBefore(textChild, tagChild)
            xml = domObj.toxml()
            # modify the xml such that MAE can read it
            prettyXml = "<?xml version=\"1.0\" ?>\n<annotationScheme>\n"
            for line in xml.split("\n")[1:]:
                if line.strip()=="":
                    continue
                elif line == "]]></TEXT><TAGS>":
                    prettyXml += "]]></TEXT>\n<TAGS>\n"
                else:
                    prettyXml += line + "\n"
                    
            path = targetDir + "/" + filename
            if subset == "aesop":
                path += ".xml"
            with codecs.open(path, "w", encoding="utf-8") as file:
                print(filename)
                file.write(prettyXml)
            



if __name__ == '__main__':
    
    sourceDataAesop = sys.argv[1]
    if not sourceDataAesop.endswith("TAC_2011_AESOP_Test_Data"):
        if not sourceDataAesop.endswith("/"):
            sourceDataAesop += "/"
        sourceDataAesop += "TAC_2011_AESOP_Test_Data"
    print("Your TAC_2011_AESOP_Test_Data location: ", sourceDataAesop)
    
    sourceDataGflow = sys.argv[2]
    if not sourceDataGflow.endswith("final-summaries"):
        print("Please give the path to 'final-summaries' folder of the G-Flow data set!")
    if not (sourceDataGflow.endswith("final-summaries") or sourceDataGflow.endswith("final-summaries/")):
        if not sourceDataGflow.endswith("/"):
            sourceDataGflow += "/"
        sourceDataGflow += "final-summaries"
    print("Your G-Flow summaries location: ", sourceDataGflow)
    
    # Run the merging process
    mergeDataAndAnnotations(sourceDataAesop + "/data", "../data/stand-off/annotations_TAC", "../data/complete/TAC2011", "aesop")
    mergeDataAndAnnotations(sourceDataGflow, "../data/stand-off/annotations_G-Flow", "../data/complete/G-flow", "gflow")
